gnuplot3d = function (X,Y,Z,W,minX=1,maxX=5,minY=0,maxY=5,minZ=0,maxZ=5,labelX="X",labelY="Y",labelZ="Z", ticsX="", ticsY="", ticsZ="", f="x + y", title="Default Title", exportSvg = FALSE) {
  #export data to gnuplot file
  outFile = file("./tmp/gnuplot3d.data", "w")
  cat("#",labelX,labelY,labelZ,"\n",sep=" ")
  write.table(data.frame(X,Y,Z), file = outFile , row.names = FALSE, col.names = FALSE)
  close(outFile)
   
  #export gnuplot script
  outFile = file("./tmp/gnuplot3d.script", "w")
  cat(file = outFile, sep = "", "set xrange [",minX, ":",maxX,"]\n")
  cat(file = outFile, sep = "", "set yrange [",minY, ":",maxY,"]\n")
  cat(file = outFile, sep = "", "set zrange [",minZ, ":",maxZ,"]\n")
  cat(file = outFile, sep="", "set xlabel \"",labelX,"\"\n")
  cat(file = outFile, sep="", "set ylabel \"",labelY,"\"\n")
  cat(file = outFile, sep="", "set zlabel \"",labelZ,"\"\n")  

  if(ticsX != ""){
    cat(file = outFile, sep="", "set xtics ",ticsX,"\n")
  }

  if(ticsY != ""){
    cat(file = outFile, sep="", "set ytics ",ticsY,"\n")
  }

  if(ticsZ != ""){
    cat(file = outFile, sep="", "set ztics ",ticsZ,"\n")
  }

  cat(file=outFile, "set logscale zx\n",sep="")  

  
  cat(file = outFile, sep="", "unset key\n")
  cat(file = outFile, sep="", "set title \"", title, "\"\n")
  cat(file = outFile, sep="", "min(x,y)=(x<y)?x:y\n")
  cat(file = outFile, sep="", "f(x,y) = ",f, "\n")

  if(exportSvg) {
    cat(file = outFile, sep = "", "set term svg \n")
    cat(file = outFile, sep = "", "set output './tmp/gnuplot3d.svg'\n")
  }

  cat(file = outFile, "splot './tmp/gnuplot3d.data' u 1:2:3 ls 7 lt rgb \"#000000\" t 'GTNetS' \n")

  close(outFile)

  #call gnuplot and present 3D data
  system("gnuplot -persist ./tmp/gnuplot3d.script")
}

plotDoubleGraphWithCI = function(X, Y, minY, maxY, Z, minBottomGraph = 0, maxBottomGraph = 0, ticsBottomGraph=0, minX=0, maxX=0, ticsX=0, minTopGraph = 0, maxTopGraph = 0, ticsTopGraph=0, orderBy=1, titleTop="Top graph", titleBottom="Bottom graph", xTitle=0, yTitle=0, yLabel=0, exportSvg=FALSE){

  tmpFrame = data.frame(as.numeric(X), as.numeric(Y), as.numeric(minY), as.numeric(maxY), as.numeric(Z))
  
  dataFileName = "tmp/doubleGraphWithCI.data"
  write.table(file=dataFileName, row.names=FALSE, col.names=FALSE, tmpFrame)
  outFile = file("tmp/doubleGraphWithCI.script", "w")

  if(exportSvg) {
    cat(file = outFile, sep = "", "set terminal svg enhanced size 640 480 fname \"Arial\" fsize 15 \n")
    cat(file = outFile, sep = "", "set output './tmp/doubleGraphWithCI.svg'\n")
  }

  
  cat(file=outFile, "unset key  \n",sep="")

  if((minX != 0) && (maxX != 0)){
    cat(file=outFile, "set xrange [",minX,":",maxX,"]\n",sep="")
  }

  if(ticsX != 0){
    cat(file=outFile, "set xtics ",ticsX,"\n",sep="")
  }

  cat(file=outFile, "set lmargin 10\n",sep="")
  cat(file=outFile, "set rmargin 2\n",sep="")
  cat(file=outFile, "set multiplot\n",sep="")
  cat(file=outFile, "set logscale x\n",sep="")  

  if(xTitle != 0){
    cat(file=outFile, "set xlabel \"",xTitle,"\" \n",sep="")
  }

  if(yLabel != 0){
    cat(file=outFile, "set ylabel \"",yLabel,"\" \n",sep="")
  }
  
  cat(file=outFile, "set size 1,0.4\n",sep="")
  cat(file=outFile, "set origin 0.0,0.0\n",sep="")

  if(minBottomGraph != 0 && maxBottomGraph !=0){
    cat(file=outFile, "set yrange [",minBottomGraph,":",maxBottomGraph,"]\n",sep="")
  }
  
  if(ticsBottomGraph != 0){
    cat(file=outFile, "set ytics ",ticsBottomGraph,"\n",sep="")
  }

  cat(file=outFile, "set tmargin 1\n",sep="")
  cat(file=outFile, "plot '",dataFileName,"' u 1:5 w points lt rgb \"#000000\" , 7000\n",sep="")
#  cat(file=outFile, "unset key\n",sep="")
  cat(file=outFile, "set grid\n",sep="")
  cat(file=outFile, "set size 1,0.6\n",sep="")

  if(yTitle != 0){
      cat(file=outFile, "set ylabel \"",yTitle,"\" \n",sep="")
  }

  if( maxTopGraph !=0){
    cat(file=outFile, "set yrange [",minTopGraph,":",maxTopGraph,"]\n",sep="")
  }
  

  if(ticsTopGraph != 0){
    cat(file=outFile, "set ytics ",ticsTopGraph,"\n",sep="")
  }

  
  cat(file=outFile, "set grid\n",sep="")
  cat(file=outFile, "unset xtics\n",sep="")
  cat(file=outFile, "set logscale xy\n",sep="")  
  
  cat(file=outFile, "set origin 0.0,0.4\n",sep="")
  cat(file=outFile, "set format x \"\"\n",sep="")
  cat(file=outFile, "set xlabel \"\"\n",sep="")
  cat(file=outFile, "plot '",dataFileName,"' u 1:2:3:4 w yerrorbars lt rgb \"#000000\" t \"",titleBottom,"\" , 7000/x t 'f(x)  = 7000/x'\n",sep="")
  cat(file=outFile, "set nomultiplot\n",sep="")
  close(outFile)
  system("gnuplot -persist tmp/doubleGraphWithCI.script")
}


t = gloFrame[((gloFrame$Model == 'GTNets')),]
gnuplot3d(X=t$NOut, Y=t$NIn,  Z=t$Mean, minX=1, maxX=100, minY=0.1, maxY=21, minZ=10, maxZ=8001)


t = gloFrame[((gloFrame$Model == 'GTNets')&(gloFrame$NIn == 0)),]
plotDoubleGraphWithCI(t$NOut, t$Mean, t$Mean-t$Error, t$Mean+t$Error, t$Sum, minX=0.9, maxX=120, minBottomGraph = 1, maxBottomGraph = 8000,  ticsTopGraph="(70, 1750, 7000)", ticsBottomGraph="(\"100 \" 7000 , \"50 \" 3500 )", ticsX="(1,2,3,4,5, 10, 20 , 30 ,40 ,50 ,100)", exportSvg=TRUE)

t = gloFrame[((gloFrame$Model == 'GTNets')&(gloFrame$NIn == 5)),]
plotDoubleGraphWithCI(t$NOut, t$Mean, t$Mean-t$Error, t$Mean+t$Error, t$Sum, minX=0.9, maxX=120, minBottomGraph = 1, maxBottomGraph = 15000,  ticsTopGraph="(\"56 kbit/s\" 7000, \"1/5\" 1400)", ticsBottomGraph="(\"100 \" 14000 , \"50 \" 7000 )", ticsX="(1,2,3,4,5, 10, 20 , 30 ,40 ,50 ,100)", exportSvg=TRUE)

t = gloFrame[((gloFrame$Model == 'GTNets')&(gloFrame$NIn == 10)),]
plotDoubleGraphWithCI(t$NOut, t$Mean, t$Mean-t$Error, t$Mean+t$Error, t$Sum, minX=0.9, maxX=120, minBottomGraph = 1, maxBottomGraph = 15000,  ticsTopGraph="(\"56 kbit/s\" 7000, \"1/10\" 700)", ticsBottomGraph="(\"100 \" 14000 , \"50 \" 7000 )", ticsX="(1,2,3,4,5, 10, 20 , 30 ,40 ,50 ,100)", exportSvg=TRUE)



t = gloFrame[((gloFrame$Model == 'LV08FD')&(gloFrame$NIn == 5)),]
plotDoubleGraphWithCI(t$NOut, t$Mean, t$Mean-t$Error, t$Mean+t$Error, t$Sum, minX=0.9, maxX=120, minBottomGraph = 1, maxBottomGraph = 15000,  ticsTopGraph="(\"56 kbit/s\" 7000, \"1/5\" 1400)", ticsBottomGraph="(\"100 \" 14000 , \"50 \" 7000 )", ticsX="(1,2,3,4,5, 10, 20 , 30 ,40 ,50 ,100)", exportSvg=FALSE)

t = gloFrame[((gloFrame$Model == 'GTNets')&(gloFrame$NIn == 10)),]
plotDoubleGraphWithCI(t$NOut, t$Mean, t$Mean-t$Error, t$Mean+t$Error, t$Sum, minX=0.9, maxX=120, minBottomGraph = 1, maxBottomGraph = 15000,  ticsTopGraph="(\"56 kbit/s\" 7000, \"1/10\" 700)", ticsBottomGraph="(\"100 \" 14000 , \"50 \" 7000 )", ticsX="(1,2,3,4,5, 10, 20 , 30 ,40 ,50 ,100)", exportSvg=FALSE)






##################################
##################################
##################################
# Graphs and numbers to support
# main text
##################################
##################################
##################################
 
#############################################################################
# Plot a 3D graph with T = f(S,L) for a fixed bandwidth of 56 kbit/s (7000 Bps)
# ./figures/ch6-02.pdf
#############################################################################
oneLinkResult = read.table("./dat/raw-1-40.data")
tb = oneLinkResult
len = length(unique(tb$NOut));

NIn      = seq(0, length=len, by=0)
NOut     = seq(0, length=len, by=0)
Model    = seq(0, length=len, by=0)
Sum      = seq(0, length=len, by=0)
Mean     = seq(0, length=len, by=0)
Median   = seq(0, length=len, by=0)
Min      = seq(0, length=len, by=0)
Max      = seq(0, length=len, by=0)
Error    = seq(0, length=len, by=0)


rm(gloFrame)
gloFrame = data.frame(NIn, NOut, Model, Sum, Mean, Median, Min, Max, Error)
glFrameIndex = 0
for(NIn in unique (tb$NIn) ){
  for (NOut in unique(tb$NOut) ){
    for (model in c('LV08PP', 'LV08FD', 'GTNets') ){      
      bw = tb[((tb$NIn == NIn)&(tb$NOut == NOut)&(tb$Model == model)),]$BandwidthShare
      glFrameIndex = glFrameIndex + 1
      
      gloFrame[glFrameIndex,1]  = NIn
      gloFrame[glFrameIndex,2]  = NOut
      gloFrame[glFrameIndex,3]  = model

      gloFrame[glFrameIndex,4]  = sum(bw)


      if(length(bw) == 1){
        gloFrame[glFrameIndex,5]  = bw
        gloFrame[glFrameIndex,6]  = bw
        gloFrame[glFrameIndex,7]  = bw
        gloFrame[glFrameIndex,8]  = bw
        gloFrame[glFrameIndex,9]  = 0
      }else{
        gloFrame[glFrameIndex,5]  = mean(bw)
        gloFrame[glFrameIndex,6]  = median(bw)
        gloFrame[glFrameIndex,7]  = min(bw)
        gloFrame[glFrameIndex,8]  = max(bw)   

        m = mean(bw)
        s = sd(bw)
        n = length(bw)
        error = qt(0.975,df=n-1)*s/sqrt(n)

        gloFrame[glFrameIndex,9]  = error
      }
    }
  }
}

             
gloFrame[((gloFrame$Model == 'LV08PP')&(gloFrame$NIn == 0)),]
gloFrame[((gloFrame$Model == 'GTNets')&(gloFrame$NIn == 0)),]
gloFrame[((gloFrame$Model == 'LV08FD')&(gloFrame$NIn == 0)),]
 




plotGnuplot2d = function(A,B,minX=1,maxX=100,minY=0,maxY=1,ticsX="", ticsY="", labelX="X",labelY="Y",title="",exportSvg = FALSE) {
  #export data to gnuplot file
  outFile = file("./tmp/gnuplot2d.data", "w")
  write.table(data.frame(A,B), file = outFile , row.names = FALSE, col.names = FALSE)
  close(outFile)

  #export gnuplot script
  outFile = file("./tmp/gnuplot2d.script", "w")

  if(exportSvg) {
    cat(file = outFile, sep = "", "set term svg \n")
    cat(file = outFile, sep = "", "set output './tmp/gnuplot2d.svg'\n")
  }

  
  cat(file = outFile, sep = "", "set grid\n")
  cat(file = outFile, sep = "", "set xrange [",minX, ":",maxX,"]\n")
  cat(file = outFile, sep = "", "set yrange [",minY, ":",maxY,"]\n")

  if(ticsX != ""){
    cat(file = outFile, sep="", "set xtics ",ticsX,"\n")
  }

  if(ticsY != ""){
    cat(file = outFile, sep="", "set ytics ",ticsY,"\n")
  }

  cat(file = outFile, sep="", "set xlabel \"",labelX,"\"\n")
  cat(file = outFile, sep="", "set ylabel \"",labelY,"\"\n")
  cat(file = outFile, sep="", "unset key\n")

  cat(file = outFile, "plot './tmp/gnuplot2d.data' w points ls 1 lt rgb \"#000000\" t '",title,"'\n")
  close(outFile)

  #call gnuplot
  system("gnuplot -persist ./tmp/gnuplot2d.script")
}

oneLinkResult = read.table("./dat/raw-1-40.data")
tb = oneLinkResult
len = length(row.names(tb[(tb$Model == 'GTNets'),]))

NFlows           = seq(0, length=len, by=0)
ErrorLV08PP      = seq(0, length=len, by=0)
ErrorLV08FD      = seq(0, length=len, by=0)

rm(gloFrame2)
gloFrame2 = data.frame(NFlows, ErrorLV08PP, ErrorLV08FD)
glFrameIndex = 0

for( row in row.names(tb[(tb$Model == 'GTNets'),]) ){

      NIn = tb[(row.names(tb) == row),]$NIn
      NOut = tb[(row.names(tb) == row),]$NOut
      fid = tb[(row.names(tb) == row),]$FlowID
  
      errPP = abs(log(tb[((tb$NIn == NIn)&(tb$NOut == NOut)&(tb$FlowID == fid)&(tb$Model == 'GTNets')),]$BandwidthShare) - log(tb[((tb$NIn == NIn)&(tb$NOut == NOut)&(tb$FlowID == fid)&(tb$Model == 'LV08PP')),]$BandwidthShare))
      
      errFD = abs(log(tb[((tb$NIn == NIn)&(tb$NOut == NOut)&(tb$FlowID == fid)&(tb$Model == 'GTNets')),]$BandwidthShare) - log(tb[((tb$NIn == NIn)&(tb$NOut == NOut)&(tb$FlowID == fid)&(tb$Model == 'LV08FD')),]$BandwidthShare))
      
      tot = tb[((tb$NIn == NIn)&(tb$NOut == NOut)&(tb$FlowID == fid)&(tb$Model == 'GTNets')),]$NIn - tb[((tb$NIn == NIn)&(tb$NOut == NOut)&(tb$FlowID == fid)&(tb$Model == 'GTNets')),]$NOut
      
      glFrameIndex = glFrameIndex + 1
      
      gloFrame2[glFrameIndex,1]  =  tot
      gloFrame2[glFrameIndex,2]  = errPP 
      gloFrame2[glFrameIndex,3]  = errFD
    }


plotGnuplot2d(A=gloFrame2$NFlows, B=gloFrame2$ErrorLV08PP, minX=-101, maxX=20, minY=0, maxY=5, labelX="Df - Uf", labelY="Error", exportSvg=TRUE)
plotGnuplot2d(A=gloFrame2$NFlows, B=gloFrame2$ErrorLV08FD, minX=-101, maxX=20, minY=0, maxY=5, labelX="Df - Uf", labelY="Error", exportSvg=TRUE)

  


abs(log(oneLinkResult[((oneLinkResult$NIn == 1)&(oneLinkResult$NOut == 20)&(oneLinkResult$Model == 'GTNets')),]$BandwidthShare) - log(oneLinkResult[((oneLinkResult$NIn == 1)&(oneLinkResult$NOut == 20)&(oneLinkResult$Model == 'LV08FD')),]$BandwidthShare))
