#!/usr/bin/perl -w
# Author        : Pedro Velho
# last modified : 10/12/2008

$workingdir="/home/velho/usr/simgrid-3.4.1/examples/msg/gtnets";
if ( -d $workingdir ){
    $thisdir=`pwd`;
    chomp($thisdir);
}else{
    print "Local working directory doesn't exist!\n";
    print "Trying grid5000...\n";

    $workingdir="/home/grenoble/pvelho/usr/simgrid-3.4.1/examples/msg/gtnets";
    
    if ( -d $workingdir ){
	$thisdir="/home/grenoble/pvelho/03-fullduplex/a-simu/";
    }else{
	print "Could not find working directory. Aborting...\n";
	exit 1;
    }
}

# ****** 1- Flows Ongoing   (FO) ******
@flowsOngoing = ("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "27", "29", "32", "34", "38", "40", "44", "51", "57", "63", "70", "73", "78", "84", "89", "95", "100");


# ****** 2- Flows Incomming (FI) ******
@flowsIncomming = ("0", "1", "2", "5", "8", "10", "12", "20");


# ****** 4- Model (M) ******
@model_list=("LV08PP", "LV08FD", "GTNets");

# ****** 5- Maximum Window (Wmax) ******
$TCP_window=65535;

#########################################
# Parse and verify arguments range
#########################################
sub parseParameters() {
    #adjust to perl parameters (arrays are from 0 up size-1)
    $task_begin=$ARGV[1]-1;
    $task_end=$ARGV[2]-1;
    if($task_begin < 0){
	print "Starting from a not valuable array index aborting...\n";
	print "Check the correctness of first parameter.\n";
	exit 1;
    }
    print "File list size is : ",scalar(@flowsOngoing),"\n";
    if($task_end >= scalar(@flowsOngoing)){
	print "Error doing more tasks than we have to do aborting...\n";
	print "Check the correctness of second parameter.\n";
	exit 1;
    }
    if($task_begin > $task_end){
	print "Error starting point is greater then end point...\n";
	print "Second parameter must be greater.\n";
	exit 1;
    }
}




#########################################
# Print error and exit
#########################################
sub printError {
    my($errorMsg) = $_[0];
    print "Error : ".$errorMsg."\n";
    print "Parse usage:\n";
    print "\tsweep_analyze parse <task_begin> <task_end>\n\n";
    print "Sweep usage:\n";
    print "\tsweep_analyze sweep <task_begin> <task_end>\n\n";
    print "Examples\n";
    print "\tsweep_analyze 1 1 # executes task 1 (fix bandwidth[0] parameter)\n";
    print "\tsweep_analyze 1 3 # executes task from 1 up to 3\n\n";
    print "\nIn this program you have ".scalar(@flowsOngoing)." tasks to parallelize.\n\n";
    exit 1;
}

if(scalar(@ARGV) < 1){ 
    printError("Needs at least one parameter sweep or parse");
}

if($ARGV[0] ne "sweep" && $ARGV[0] ne "parse"){
    printError("First argument should be sweep or parse");
}

if($ARGV[0] eq "sweep"){
    if(scalar(@ARGV) != 3){
	printError("Option sweep must be followed by two integers in task range.");
    }else {
	executeApplication();
    }
}

if($ARGV[0] eq "parse"){
    if(scalar(@ARGV) != 3){
	printError("Option parse must be followed by two integers in task range.");
    }else {
	parseParameters();
	parseResults(($task_begin+1), ($task_end+1));
    }
}






#########################################
# Execute application only once
#########################################
sub executeApplication {
    parseParameters();
    
    print "Changing working directory to ".$workingdir."\n";
    chdir($workingdir);

    open(LOGOUTPUT, ">" , $thisdir."/log/trace-file-".($task_begin+1)."-".($task_end+1).".log") ||
	die "Sorry, couldn't open file ".$thisdir."/log/trace-file-".($task_begin+1)."-".($task_end+1).".log for writting...\n";

    print LOGOUTPUT "\n";


    my($platform_file) = $thisdir."/onelink-p-template.xml";

    $it=0;
    for($it=$task_begin; $it <= $task_end; $it++) 
    {
	$flowO = $flowsOngoing[$it];
	print ">=============================================================<\n";
	print "========> Ongoing flows (O) : ",$flowO," \n";
	foreach $flowI (@flowsIncomming)
	{
	    print "========> Incomming flows (O) : ",$flowI," \n";

	    foreach $model (@model_list)
	    {
		print "========> Model     (M) : ",$model,"\n";

		# Log some stuff into the file to know used parameters
		print LOGOUTPUT ">==================================================<\n";
		print LOGOUTPUT "========> Ongoing   (O) : ",$flowO," \n";
		print LOGOUTPUT "========> Incomming (I) : ",$flowI," \n";
		print LOGOUTPUT "========> Model     (M) : ",$model,"\n";
		    		

		$temp_dep_file    = "";
		$temp_dep_file    = $thisdir."/deploy/onelink-d-na-".$flowO."-nb-".$flowI.".log";
		    
		createDeploy($flowO, $flowI, $temp_dep_file);


		# chose command depending on model
		if( $model =~ m/^LV08PP$/ ){
		    $alpha=13.0076191103735;
		    $beta=0.969832987216181;
		    $s=20537.14;
		    $cmd=$workingdir."/gtnets ".$platform_file." ".$temp_dep_file." --cfg=network/model:LV08 --cfg=fullduplex:0 --cfg=TCP_gamma:".$TCP_window." --cfg=network/latency_factor:".$alpha." --cfg=network/bandwidth_factor:".$beta." --cfg=network/weight_S:".$s." 2>&1";
		}elsif($model =~ m/LV08FD/ ){
		    $alpha=13.0076191103735;
		    $beta=0.969832987216181;
		    $s=20537.14;
		    $cmd=$workingdir."/gtnets ".$platform_file." ".$temp_dep_file." --cfg=network/model:LV08 --cfg=fullduplex:1 --cfg=TCP_gamma:".$TCP_window." --cfg=network/latency_factor:".$alpha." --cfg=network/bandwidth_factor:".$beta." --cfg=network/weight_S:".$s." 2>&1";
		}elsif( $model =~ m/^GTNets$/ ){
		    $cmd=$workingdir."/gtnets ".$platform_file." ".$temp_dep_file." --cfg=network/model:".$model." --cfg=fullduplex:0 --cfg=TCP_gamma:".$TCP_window." 2>&1";
		}	

		
		# Run with parameters
		print("Running command:".$cmd."\n");
		
		# Open temporary output file
		open(TMPOUTPUT, $cmd." | ") || die "Sorry, could't run application...\n";
		
		# Copy execution trace into global trace file
		while(<TMPOUTPUT>)
		{
		    my $line = $_;
		    chomp $line;
		    print LOGOUTPUT $line."\n";
		}
		print LOGOUTPUT "=========================><=========================\n";
	    }
	}
	print "==============================><=============================\n";
    }
    close(LOGOUTPUT);
}



#########################################
# Parse resulting values
# Genarate input file for R
#########################################
sub parseResults {
    my($begin_it, $end_it) = @_;
    if($begin_it == $end_it){
	open(RAWOUTPUT, ">".$thisdir."/dat/raw-".$begin_it.".data") ||
	    die "Sorry, couldn't open file ".$thisdir."/dat/raw-".$it.".data for writting...\n";
    }else{
	open(RAWOUTPUT, ">".$thisdir."/dat/raw-".$begin_it."-".$end_it.".data") ||
	    die "Sorry, couldn't open file ".$thisdir."/dat/raw-".$begin_it."-".$end_it.".data for writting...\n";
    }
;

    print RAWOUTPUT " NIn NOut Model FlowID BandwidthShare CommRemaining\n";


    my($counter,$it);

    $it=1;
    $counter=1;
    for($it = $begin_it ; $it <= $end_it ; $it++) 
    {	
	$tracefile = $thisdir."/log/"."trace-file-".($it)."-".($it).".log";

	unless (-e $tracefile) {
	    print "File $tracefile doesn't exist, skiping...\n";
	    next;
	} 
	open(LOGOUTPUT, "<", $tracefile) || die "Sorry, I coundn't open file ".$tracefile." for reading...";
	print $tracefile."\n";

	my($formater);
	my($NIn, $NOut, $Model, $FlowID, $BandwidthShare, $CommRemaining);


	$latency_limited = 0;
	# Copy execution trace into global trace file
	while(<LOGOUTPUT>)
	{
	    my $line = $_;
	    chomp $line;
	    
	    if($line =~ m/^$/){
		print "Blank line found, skipping...\n";
	    }

	    # Detect latency limited
	    if($line =~ /WARN/){
		$latency_limited = 1;
		next;
	    }

	    # Parse a flow
	    if($line =~ m/.*FLOW\[(\d+)\].*message\sfrom\s(.*)\sto\s(.*)\s*with.*/){
		$FlowID = $1;		

		if($FlowID =~ m/^$/){
		    print "Null strings, line is:\n";
		    print $line."\n";
		    print " FlowID=".$FlowID." from=".$2." to=".$3."\n";
		}
		
		my($BandwidthSh,$CommR)=split(/;/, $line);
		$BandwidthShare = get_value($BandwidthSh);
		$CommRemaining  = get_value($CommR);

		if( $BandwidthShare =~ m/^$/ || $CommRemaining =~ m/^$/ ){
		    print "Null strings, line is:\n";
		    print $line."\n";
		    print " BandwidthSh=".$BandwidthSh." CommR=".$CommR."\n";
		}

		
		print RAWOUTPUT $counter++." ".$NIn." ".$NOut." \"".$Model."\" ".$FlowID." ".$BandwidthShare." ".$CommRemaining."\n";
		$latency_limited=0;
		next;
	    }
	    
	    # Find Number of Ongoing flows
	    if($line =~ /Ongoing/){
		$NOut = $line;
		$NOut =~ s/^.*\(O\)\s*:\s*(\d+)\s*$/$1/x;
		next;
	    }

	    # Find Number of Incomming flows
	    if($line =~ /Incomming/){
		$NIn = $line;
		$NIn =~ s/^.*\(I\)\s*:\s*(\d+)\s*$/$1/x;
		next;
	    }
	    
	    # Find model parameter
	    if($line =~ /Model/){
		$line =~ s/^.*\s+Model\s+\(M\)\s+:\s+(.*)\s*/$1/x;
		$Model = $line;
		next;
	    }

	}
	close(LOGOUTPUT);
    }
    close(RAWOUTPUT);
}

sub createDeploy {
    my($na) = shift;
    my($nb) = shift;
    my($filename) = shift;
    my($i,$offset);
    $offset = 1000;
    
    open(FILE, " > ".$filename) || die "Error: opening file ".$filename." for writing...";

    print FILE "<?xml version='1.0'?>\n";
    print FILE "<!DOCTYPE platform SYSTEM \"simgrid.dtd\">\n";
    print FILE "<platform version=\"2\">\n\n";

    # create flows from S1 to C1
    for($i=1; $i <= $na ; $i++) {
	print FILE "  <process host=\"S1\" function=\"master\">\n";
	print FILE 	"      <argument value=\"100000000\"/>\n";
	print FILE "      <argument value=\"C1\"/>\n";
	print FILE "      <argument value=\"".$i."\"/>\n";
	print FILE "  </process>\n";
	print FILE "  <process host=\"C1\" function=\"slave\">\n";
	print FILE "    <argument value=\"".$i."\"/>\n";
	print FILE "  </process>\n\n";
    }

    # create flows from C1 to S1
    for($i=1; $i <= $nb ; $i++) {
	print FILE "  <process host=\"C1\" function=\"master\">\n";
	print FILE 	"      <argument value=\"100000000\"/>\n";
	print FILE "      <argument value=\"S1\"/>\n";
	print FILE "      <argument value=\"".($i+$offset)."\"/>\n";
	print FILE "  </process>\n";
	print FILE "  <process host=\"S1\" function=\"slave\">\n";
	print FILE "    <argument value=\"".($i+$offset)."\"/>\n";
	print FILE "  </process>\n\n";
    }

    print FILE "</platform>\n";

    close(FILE);
}

#########################################
# Parse numerical value within a line
#########################################
sub get_value {
    my($line)=shift;
    $line=" $line";
    $line =~ s/^
                .*[^\d\.+-eE]+         # anything except something in a number
                (
                   [+-]?\ *          # first, match an optional sign
                   (                 # then match integers or f.p. mantissas:
                      \d+            # start out with a ...
                      (
                          \.\d*      # mantissa of the form a.b or a.
                      )?             # ? takes care of integers of the form a
                      |\.\d+         # mantissa of the form .b
                   )
                   ([eE][+-]?\d+)?  # finally, optionally match an exponent
		 )
                \D*$/$1/x;
    return $line;
}
