library(gplots)

################################################################
gnuplot3D = function (X=c(1,2,3),Y=c(1,2,3),Z=c(1,2,3),minX=0,maxX=5,minY=0,maxY=5,minZ=0,maxZ=5,labelX="X",labelY="Y",labelZ="Z") {
  #export data to gnuplot file
  outFile = file("./tmp/gnuplot3d.data", "w")
  cat("#",labelX,labelY,labelZ,"\n",sep=" ")
  write.table(data.frame(X,Y,Z), file = outFile , row.names = FALSE, col.names = FALSE)
  close(outFile)
   
  #export gnuplot script
  outFile = file("./tmp/gnuplot3d.script", "w")
  cat(file = outFile, sep = "", "set xrange [",minX, ":",maxX,"]\n")
  cat(file = outFile, sep = "", "set yrange [",minY, ":",maxY,"]\n")
  cat(file = outFile, sep = "", "set zrange [",minZ, ":",maxZ,"]\n")
  cat(file = outFile, sep="", "set xlabel \"",labelX,"\"\n")
  cat(file = outFile, sep="", "set ylabel \"",labelY,"\"\n")
  cat(file = outFile, sep="", "set zlabel \"",labelZ,"\"\n")
  cat(file = outFile, "splot './tmp/gnuplot3d.data' w points\n")
  

  close(outFile)

  #call gnuplot and present 3D data
  system("gnuplot -persist ./tmp/gnuplot3d.script")
}
 
################################################################
gnuplot2D = function (X=c(1,2,3),Y=c(1,2,3),minX=0,maxX=5,minY=0,maxY=5,labelX="X",labelY="Y", generateEps=FALSE, logScale=FALSE) {
  #export data to gnuplot file
  outFile = file("./tmp/gnuplot2d.data", "w")
  cat("#",labelX,labelY,"\n",sep=" ")
  write.table(data.frame(X,Y), file = outFile , row.names = FALSE, col.names = FALSE)
  close(outFile)
   
  #export gnuplot script
  outFile = file("./tmp/gnuplot2d.script", "w")
  cat(file = outFile, sep = "", "set xrange [",minX, ":",maxX,"]\n")
  cat(file = outFile, sep = "", "set yrange [",minY, ":",maxY,"]\n")
  cat(file = outFile, sep="", "set xlabel \"",labelX,"\"\n")
  cat(file = outFile, sep="", "set ylabel \"",labelY,"\"\n")

  if(logScale){
    cat(file = outFile, sep="", "set logscale y\n")
  }
  
  cat(file = outFile, "plot './tmp/gnuplot2d.data' w points\n")  
#  cat(file = outFile, "1.560+115.669*x\n");        
  if(generateEps){
#    cat(file = outFile, "set terminal postscript eps enhanced  color\n")
    cat(file = outFile, "set terminal png\n")

    cat(file = outFile, "set output \"./tmp/gnuplot2d.png\"\n")
    cat(file = outFile, "replot\n")  
  }

  
  close(outFile)
  
  #call gnuplot and present 3D data
  system("gnuplot -persist ./tmp/gnuplot2d.script")
}


################################################################
gnuplotCoffInterval2D = function (X=c(1,2,3),Y=c(1,2,3),minConf=c(0.8,1.4,2.7), maxConf=c(1.2,2.1,3.4), minX=0,maxX=5,minY=0,maxY=5,labelX="X",labelY="Y", generateEps=FALSE) {
  #export data to gnuplot file
  outFile = file("./tmp/gnuplot2d.data", "w")
  cat("#",labelX,labelY,"\n",sep=" ")
  write.table(data.frame(X,Y,minConf,maxConf), file = outFile , row.names = FALSE, col.names = FALSE)
  close(outFile)
   
  #export gnuplot script
  outFile = file("./tmp/gnuplot2d.script", "w")
  cat(file = outFile, sep = "", "set xrange [",minX, ":",maxX,"]\n")
  cat(file = outFile, sep = "", "set yrange [",minY, ":",maxY,"]\n")
  cat(file = outFile, sep="", "set xlabel \"",labelX,"\"\n")
  cat(file = outFile, sep="", "set ylabel \"",labelY,"\"\n")
  cat(file = outFile, "plot './tmp/gnuplot2d.data' u 1:2  t 'Mean Values' w points, ")
  cat(file = outFile, "'./tmp/gnuplot2d.data' u 1:3 t 'Min CI' w points  , ")
  cat(file = outFile, "'./tmp/gnuplot2d.data' u 1:4 t 'Max CI' w points\n")


  if(generateEps){
#    cat(file = outFile, "set terminal postscript eps enhanced  color\n")
    cat(file = outFile, "set terminal png\n")

    cat(file = outFile, "set output \"./tmp/gnuplot2d.png\"\n")
    cat(file = outFile, "replot\n")  
  }

  
  close(outFile)
  
  #call gnuplot and present 3D data
  system("gnuplot -persist ./tmp/gnuplot2d.script")
}



#####################
#Main
#####################
resultTable = read.table("./dat/raw.data") 

tmpTableGT = resultTable[(resultTable$Model == 'REAL'),]
gnuplot2D(X=tmpTableGT$LatencyY, Y=tmpTableGT$ResultBandwidthFlowA, minX=min(tmpTableGT$LatencyY), maxX=max(tmpTableGT$LatencyY), minY=1e4, maxY=1e8, labelX="Latency_Y",labelY="Bandwidth of flow A", generateEps=TRUE, logScale=TRUE)


gnuplot2D(X=tmpTableGT$LatencyY, Y=tmpTableGT$ResultBandwidthFlowA/tmpTableGT$ResultBandwidthFlowB, minX=min(tmpTableGT$LatencyY), maxX=max(tmpTableGT$LatencyY), minY=min(tmpTableGT$ResultBandwidthFlowA/tmpTableGT$ResultBandwidthFlowB), maxY=50, labelX="Latency_Y",labelY="Ratio F1/F2", generateEps=TRUE)

gnuplot2D(X=tmpTableGT$LatencyY, Y=tmpTableGT$ResultBandwidthFlowA, minX=min(tmpTableGT$LatencyY), maxX=max(tmpTableGT$LatencyY), minY=min(tmpTableGT$ResultBandwidthFlowA), maxY=max(tmpTableGT$ResultBandwidthFlowA), labelX="Latency_Y",labelY="Bw F1", generateEps=TRUE)

gnuplot2D(X=tmpTableGT$LatencyY, Y=tmpTableGT$ResultBandwidthFlowB, minX=min(tmpTableGT$LatencyY), maxX=max(tmpTableGT$LatencyY), minY=min(tmpTableGT$ResultBandwidthFlowB), maxY=max(tmpTableGT$ResultBandwidthFlowB), labelX="Latency_Y",labelY="Bw F2", generateEps=TRUE)


X=tmpTableGT$LatencyY
Y=tmpTableGT$ResultBandwidthFlowA/tmpTableGT$ResultBandwidthFlowB
summary(lm(Y ~ X))
postscript("plotmean.eps")
plotmeans(Y ~ X)
dev.off()


zp = 1.645
paramlaty = c()
xtilde = c()
stilde = c()
minConf = c()
maxConf = c()
nlen = c()
pval = c()
sdlist = c()
meanlist = c()

for(laty in unique(tmpTableGT$LatencyY)){
  #
  work = tmpTableGT[(tmpTableGT$LatencyY == laty),]$ResultBandwidthFlowA/tmpTableGT[(tmpTableGT$LatencyY == laty),]$ResultBandwidthFlowB
  paramlaty = c(paramlaty, laty)

  xt = mean(work)

  
  pval = c(pval, ks.test(work, "pnorm")$p.value)
  
  xtilde = c(xtilde, xt)
  st = sd(work)
  sdlist = c(sdlist, st)
  stilde = c(stilde, st)

  minConf = c(minConf, xt - zp* (st/(sqrt(length(work)))))
  maxConf = c(maxConf, xt + zp* (st/(sqrt(length(work)))))

  nlen = c(nlen, length(work))
}

gnuplotCoffInterval2D(X=paramlaty, Y=xtilde, minConf=minConf, maxConf=maxConf, minX=min(paramlaty), maxX=max(paramlaty), minY=min(xtilde), maxY=20, labelX="Latency", labelY="FlowA/FlowB", generateEps=TRUE)

lm(xtilde ~ paramlaty)

gnuplot2D(X=paramlaty, Y=xtilde, minX=min(paramlaty), maxX=max(paramlaty), minY=min(xtilde), maxY=50, labelX="Latency", labelY="FlowA/FlowB", generateEps=TRUE)


linefit <- lm(xtilde ~ paramlaty)
summary(linefit)
postscript("plot-conf-inter.eps")
#1600000
plotCI(x=paramlaty, y=xtilde, ui=maxConf, li=minConf, xlab='Latency', ylab='FlowA/FlowB', ylim=c(1,20), col=51, barcol=34) 
abline(linefit)
grid()
dev.off()


