plotDoubleGraphWithCI = function(X, Y, minY, maxY, Z, minBottomGraph = 0, maxBottomGraph = 0, ticsBottomGraph=0, minX=0, maxX=0, ticsX=0, minTopGraph = 0, maxTopGraph = 0, ticsTopGraph=0, orderBy=1, titleTop="Top graph", titleBottom="Bottom graph", xTitle=0, yTitle=0, yLabel=0, exportSvg=FALSE){

  tmpFrame = data.frame(as.numeric(X), as.numeric(Y), as.numeric(minY), as.numeric(maxY), as.numeric(Z))
  
  dataFileName = "tmp/doubleGraphWithCI.data"
  write.table(file=dataFileName, row.names=FALSE, col.names=FALSE, tmpFrame)
  outFile = file("tmp/doubleGraphWithCI.script", "w")

  if(exportSvg) {
    cat(file = outFile, sep = "", "set terminal svg enhanced size 640 480 fname \"Arial\" fsize 15 \n")
    cat(file = outFile, sep = "", "set output './tmp/doubleGraphWithCI.svg'\n")
  }

  
  cat(file=outFile, "unset key  \n",sep="")

  if((minX != 0) && (maxX != 0)){
    cat(file=outFile, "set xrange [",minX,":",maxX,"]\n",sep="")
  }

  if(ticsX != 0){
    cat(file=outFile, "set xtics ",ticsX,"\n",sep="")
  }

  cat(file=outFile, "set logscale xy\n",sep="")  
  cat(file=outFile, "set lmargin 10\n",sep="")
  cat(file=outFile, "set rmargin 2\n",sep="")
  cat(file=outFile, "set multiplot\n",sep="")

  if(xTitle != 0){
    cat(file=outFile, "set xlabel \"",xTitle,"\" \n",sep="")
  }

  if(yLabel != 0){
    cat(file=outFile, "set ylabel \"",yLabel,"\" \n",sep="")
  }
  
  cat(file=outFile, "set size 1,0.4\n",sep="")
  cat(file=outFile, "set origin 0.0,0.0\n",sep="")

  if(minBottomGraph != 0 && maxBottomGraph !=0){
    cat(file=outFile, "set yrange [",minBottomGraph,":",maxBottomGraph,"]\n",sep="")
  }
  
  if(ticsBottomGraph != 0){
    cat(file=outFile, "set ytics ",ticsBottomGraph,"\n",sep="")
  }

  cat(file=outFile, "set tmargin 1\n",sep="")
  cat(file=outFile, "plot '",dataFileName,"' u 1:5 w points lt rgb \"#CCCCCC\"\n",sep="")
  cat(file=outFile, "unset key\n",sep="")
  cat(file=outFile, "set grid\n",sep="")
  cat(file=outFile, "set size 1,0.6\n",sep="")

  if(yTitle != 0){
      cat(file=outFile, "set ylabel \"",yTitle,"\" \n",sep="")
  }

  if( maxTopGraph !=0){
    cat(file=outFile, "set yrange [",minTopGraph,":",maxTopGraph,"]\n",sep="")
  }
  

  if(ticsTopGraph != 0){
    cat(file=outFile, "set ytics ",ticsTopGraph,"\n",sep="")
  }

  cat(file=outFile, "set grid\n",sep="")
  cat(file=outFile, "unset xtics\n",sep="")
  cat(file=outFile, "set origin 0.0,0.4\n",sep="")
  cat(file=outFile, "set format x \"\"\n",sep="")
  cat(file=outFile, "set xlabel \"\"\n",sep="")
  cat(file=outFile, "plot '",dataFileName,"' u 1:2:3:4 w yerrorbars lt rgb \"#000000\" t \"",titleBottom,"\" , 7000/x\n",sep="")
  cat(file=outFile, "set nomultiplot\n",sep="")
  close(outFile)
  system("gnuplot -persist tmp/doubleGraphWithCI.script")
}

t = gloFrame[((gloFrame$Model == 'GTNets')&(gloFrame$NIn == 0)),]
if( t$N < 0){
  left = m
  right = m
}else{
  m = t$Mean
  s = t$Sd
  n = t$N
  error = qt(0.975,df=n-1)*s/sqrt(n)
}
plotDoubleGraphWithCI(t$NOut, t$Mean, t$Mean-error, t$Mean+error, t$Sum)




##################################
##################################
##################################
# Graphs and numbers to support
# main text
##################################
##################################
##################################

#############################################################################
# Plot a 3D graph with T = f(S,L) for a fixed bandwidth of 56 kbit/s (7000 Bps)
# ./figures/ch6-02.pdf
#############################################################################
oneLinkResult = read.table("./dat/raw-1-14.data")
tb = oneLinkResult
len = length(unique(tb$NOut));

NIn      = seq(0, length=len, by=0)
NOut     = seq(0, length=len, by=0)
Model    = seq(0, length=len, by=0)
Sum      = seq(0, length=len, by=0)
Mean     = seq(0, length=len, by=0)
Median   = seq(0, length=len, by=0)
Min      = seq(0, length=len, by=0)
Max      = seq(0, length=len, by=0)
Sd       = seq(0, length=len, by=0)
N        = seq(0, length=len, by=0)


rm(gloFrame)
gloFrame = data.frame(NIn, NOut, Model, Sum, Mean, Median, Min, Max, Sd, N)
glFrameIndex = 0
for(NIn in unique (tb$NIn) ){
  for (NOut in unique(tb$NOut) ){
    for (model in c('LV08PP', 'LV08FD', 'GTNets') ){      
      bw = tb[((tb$NIn == NIn)&(tb$NOut == NOut)&(tb$Model == model)),]$BandwidthShare
      glFrameIndex = glFrameIndex + 1
      
      gloFrame[glFrameIndex,1]  = NIn
      gloFrame[glFrameIndex,2]  = NOut
      gloFrame[glFrameIndex,3]  = model

      gloFrame[glFrameIndex,4]  = sum(bw)
      gloFrame[glFrameIndex,5]  = mean(bw)
      gloFrame[glFrameIndex,6]  = median(bw)
      
      gloFrame[glFrameIndex,7]  = min(bw)
      gloFrame[glFrameIndex,8]  = max(bw)   
 
      gloFrame[glFrameIndex,9]  = sd(bw)   
      gloFrame[glFrameIndex,10] = length(bw)   

    }
  }
}


gloFrame[((gloFrame$Model == 'LV08PP')&(gloFrame$NIn == 0)),]
gloFrame[((gloFrame$Model == 'GTNets')&(gloFrame$NIn == 0)),]
gloFrame[((gloFrame$Model == 'LV08FD')&(gloFrame$NIn == 0)),]








