/*
** client.c
** 
** Made by (Pedro Velho)
** Login   <velho@neutres>
** 
** Started on  Tue May 19 15:24:31 2009 Pedro Velho
** Last update Sun May 12 01:17:25 2002 Speed Blue
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>

int sockfd=0;

void error(char *msg)
{
  perror(msg);
  fflush(NULL);
  close(sockfd);
  exit(1);
}

int main(int argc, char *argv[])
{
  int flow_id, serv_addr_port;
  unsigned long int flow_size;
  ssize_t max_slice = 10000;
  ssize_t n;
  struct sockaddr_in serv_addr;
  struct hostent *server;
  char *buffer;

  printf("Initializing variables\n");
  fflush(NULL);
  
  sockfd = serv_addr_port = n = flow_id = flow_size= 0;
  bzero((char *) &serv_addr, sizeof(serv_addr));
  server = NULL;
 
  if (argc != 5){
    fprintf(stderr,"Error\n\tUsage: %s <hostname> <port> <flowid> <size>\n\n", argv[0]);
    exit(0);
  }  

  sscanf(argv[4], "%lu", &flow_size);
  printf("Parsed flow_size=%lu as input\n", flow_size);
  fflush(NULL);
  if(flow_size < 1){
    fprintf(stderr, "client: invalid flow_size %lu, valid > 1\n", flow_size);
    error("Aborting...");    
  }

  sscanf(argv[3], "%d", &flow_id); 
  if(flow_id < 1 || flow_id > 3){
    fprintf(stderr, "client: invalid flowid %d, valid 1 2 3\n", flow_id);
    error("Aborting...");    
  }

  sscanf(argv[2], "%d", &serv_addr_port); 
  if(serv_addr_port <= 1024){
    fprintf(stderr, "client: invalid port: %d, MUST > 1024\n", serv_addr_port);
    error("Aborting...");    
  }

  sockfd = socket(AF_INET, SOCK_STREAM, 0);

  printf("Initializing socket\n");
  fflush(NULL);
  
  if (sockfd < 0){
    fprintf(stderr, "client: opening socket\n");
    error("Aborting...");
  }

  printf("Discovering host address...\n");
  fflush(NULL);

  server = gethostbyname(argv[1]);
  if (server == NULL){
    fprintf(stderr,"client: no such host\n");
    error("Aborting...");  
  }
  
  serv_addr.sin_family = AF_INET;
  bcopy((char *)server->h_addr,
	(char *)&serv_addr.sin_addr.s_addr,
	server->h_length);
  serv_addr.sin_port = htons(serv_addr_port);

  printf("Found host address:%s\n", inet_ntoa((struct in_addr)serv_addr.sin_addr));
  fflush(NULL);
  
  if (connect(sockfd, (const struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0){
    fprintf(stderr, "client: connecting\n");
    error("Aborting...");
  }

  /*
   * Send flowID
   */
  n = send(sockfd, &flow_id, sizeof(int), 0);
  if (n < 0){
    fprintf(stderr, "client: sending flow_id");
    error("Aborting...");
  }

  /*
   * Send flowSIZE
   */
  n = send(sockfd, &flow_size, sizeof(unsigned long int), 0);
  if (n < 0){
    fprintf(stderr, "client: sending flow_size");
    error("Aborting...");
  }

  if(max_slice < flow_size){
    buffer = (char *)calloc(max_slice, 1);
    memset(buffer, 'P', max_slice);
  }else {
    buffer = (char *)calloc(flow_size, 1);
    memset(buffer, 'P', flow_size);
  }
  

  while(flow_size > 0){
    if(max_slice < flow_size){
      n = send(sockfd,buffer,max_slice,0);
    }else {
      n = send(sockfd,buffer,flow_size,0);
    }
    if (n == -1){
      fprintf(stderr, "client: sending message %lu bytes long!", flow_size);
      error("Aborting...");
    }
    flow_size -= n;
    //printf("Sended %d Bytes to server\n", (int) n);
    fflush(NULL);
  }

  close(sockfd);
  
  return 0;
}
  
