/*
** server.c
** 
** Made by (Pedro Velho)
** Login   <velho@neutres>
** 
** Started on  Tue May 19 15:23:58 2009 Pedro Velho
** Last update Sun May 12 01:17:25 2002 Speed Blue
*/
#define _REENTRANT
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <sys/uio.h>
#include <unistd.h>
#include <pthread.h>
#include <semaphore.h>
#include <sys/time.h>
#include <time.h>
#include <arpa/inet.h>
#include <netdb.h>


/* function prototypes and global variables */
void *do_chld(void *);

void error(char *msg){
  perror(msg);
  exit(1);
}

sem_t gl_mutex;
sem_t gl_barrier;
unsigned long int gl_remaining[3];
int gl_index[3];
int gl_socket[4];
unsigned int gl_barrier_flag=0;

struct thread_param {
  int index;
  int sockfd;
};

int main(int argc, char **argv)
{
  int sockfd, newsockfd, i;
  void *return_value; 
  socklen_t clilen =0;
  struct sockaddr_in cli_addr, serv_addr;
  pthread_t chld_thr[3];
  int cli_addr_port;
  struct thread_param param[3];
  struct hostent *server;
 
  i = sockfd = newsockfd = clilen = -1;
  return_value = NULL;
  memset((char *) &serv_addr, 0, sizeof(cli_addr));    
  memset((char *) &serv_addr, 0, sizeof(serv_addr));
  cli_addr_port = -1;

  if(argc < 3){
    fprintf(stderr, "server: missing parameter\n");
    fprintf(stdout, "\tUsage: %s <interface> <port>\n\n", argv[0]);
    error("Aborting...");    
  }else if(argc > 3) {
    for(i=0; i<argc; i++){
      if(!strcmp(argv[i], "--barrier")){
	gl_barrier_flag = 1;
      }
    }

    if(!gl_barrier_flag){
      fprintf(stderr, "server: the only availables options are:\n\t'--barrier'\n\tWait three conections and start all at same time, using barrier.\n");
      error("Aborting...");
    }
  }

  

  if((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0){
    fprintf(stderr, "server: can't open stream socket\n");
    error("Aborting...");    
  }
  
  sscanf(argv[2], "%d", &cli_addr_port);  
  if(cli_addr_port <= 1024){
    fprintf(stderr, "server: invalid port: %d, MUST > 1024\n", cli_addr_port);
    error("Aborting...");    
  }

  printf("server: initializing address\n");
  fflush(NULL);

  server = gethostbyname(argv[1]);
  if (server == NULL){
    fprintf(stderr,"client: no such host\n");
    error("Aborting...");  
  }
  
  serv_addr.sin_family = AF_INET;
  bcopy((char *)server->h_addr, (char *)&serv_addr.sin_addr.s_addr, server->h_length);
  serv_addr.sin_port = htons(cli_addr_port);
	
  if(bind(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0){
    fprintf(stderr,"server: can't bind local address\n");
    exit(0);
  }

  sem_init(&gl_mutex, 0, 1); 
  for(i=0;i<3;i++){
    gl_socket[i] = 0;
    gl_remaining[i] = 0;
  }

  /* set the level of thread concurrency we desire */
  listen(sockfd, 3);

  /* assign main server socket to array */
  gl_socket[3] = sockfd;

  for(i=0;i<3;i++){

    printf("server: blocking waiting connection on %s, port %d\n", inet_ntoa((struct in_addr)serv_addr.sin_addr), cli_addr_port);
    fflush(NULL);

    clilen = sizeof(cli_addr);
    newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, &clilen);
    
    if(newsockfd < 0){
      fprintf(stderr,"server: accept error\n");
      error("Aborting...");    
    } else {
      fprintf(stderr,"server: got connection from %s\n", inet_ntoa(cli_addr.sin_addr) );
      fflush(NULL);
    }
  


    param[i].index = i;
    param[i].sockfd = newsockfd;

    gl_socket[i] = newsockfd;

    /* create a new thread to process the incomming request */
    pthread_create(&chld_thr[i], NULL, do_chld, (void *) &param[i]);

    /* the server is now free to accept another socket request */
  }

  for(i=0;i<3;i++){  
    sem_post(&gl_barrier);
  }


  for(i=0;i<3;i++){  
    /*
     * should wait until all threads have finished
     */
    pthread_join(chld_thr[i], &return_value);
    if( return_value != 0){
      fprintf(stderr,"server: accept error\n");
      error("Aborting...");    
    }
  }

  close(sockfd);
  return(0);
}


/* 
 *    this is the routine that is executed from a new thread 
 */
void *do_chld(void *arg)
{
  int 	mysocfd = ((struct thread_param *) arg)->sockfd;
  unsigned long int flow_size;
  unsigned long int total_size;
  int i=0;
  int flow_id=0;
  char *data;
  ssize_t max_slice = 10000;
  int index = ((struct thread_param *) arg)->index; 
  ssize_t n;
  struct timeval start,finish;
  double duration;
  double bandwidth=0;
  
  start.tv_sec = finish.tv_sec =  0;
  start.tv_usec = finish.tv_usec = 0;
  total_size = 0;

  n = 0;



  printf("Child thread [%p]: Socket number = %d\n", (void *)pthread_self(), mysocfd);



  /* read flow id */
  printf("Child thread [%p]: Receiving flow_id\n", (void *)pthread_self());
  n = recv(mysocfd, (int *)&flow_id, sizeof(int), 0);
  if (n == -1){
    fprintf(stderr, "Child thread [%p]: server: receive message %d bytes long!", (void *)pthread_self(), (int)n);
    error("Aborting...");
  }

  printf("Child thread [%p]: Registering flow_id: %d\n", (void *)pthread_self(), flow_id);
  gl_index[index] = flow_id; 


  /* read flow size */
  printf("Child thread [%p]: Receiving flow_size\n", (void *)pthread_self());
  n = recv(mysocfd, (unsigned long int *)&flow_size, sizeof(unsigned long int), 0);
  if (n == -1){
    fprintf(stderr, "Child thread [%p]: server: receive message %d bytes long!", (void *)pthread_self(), (int)n);
    error("Aborting...");
  }

  printf("Child thread [%p]: Registering flow_size: %lu\n", (void *)pthread_self(), flow_size);
  total_size = gl_remaining[index] = flow_size;

  if(max_slice < flow_size){
    data = (char *)calloc(max_slice, 1);
    data = memset(data, '\0', max_slice);
  }else {
    data = (char *)calloc(flow_size, 1);
    data = memset(data, '\0', flow_size);
  }

  if(gl_barrier_flag){
    printf("Child thread [%p]: Blocking on barrier\n", (void *)pthread_self());
    fflush(NULL);
    sem_wait(&gl_barrier);
  }

  printf("Child thread [%p]: Starting timer...\n", (void *)pthread_self());
  gettimeofday(&start, NULL);  

  /* read from the given socket */
  while(flow_size > 0){

    if(max_slice < flow_size){
      n = recv(mysocfd, (void *)data,  max_slice, 0);
    }else {
      n = recv(mysocfd, (void *)data,  flow_size, 0);
    }

    if (n == -1){
      fprintf(stderr, "Child thread [%p]: server: receive message %d bytes long!", (void *)pthread_self(), (int)n);
      error("Aborting...");
    }
    flow_size -= n;
    gl_remaining[index] -= n;

    //printf("Child thread [%p]: Received: %d Bytes, Remaining %d Bytes\n", (void *)pthread_self(), n, flow_size);
  }

  gettimeofday(&finish, NULL);

  duration = (finish.tv_sec+finish.tv_usec*1e-6) - (start.tv_sec+start.tv_usec*1e-6); 
 
  printf("Child [%p]: Done Processing...\n", (void *)pthread_self()); 

  sem_wait(&gl_mutex);  
  for(i=0; i<3; i++){
    /* only print registered flows */
    if(gl_index[i] != 0){
      bandwidth = (total_size - gl_remaining[i])/duration;
      
      printf("FLOW[%d]: remaining %lu ; total size %lu, bw %4.2f MBytes/s : %4.2f Kbits/s : %f\n", gl_index[i], gl_remaining[i], total_size, bandwidth/1000000, (bandwidth*8)/1000, bandwidth); 
    }
  }

  /* cleanup, close sockets and exit program */
  for(i=0; i<4; i++){
    if(gl_socket[i] != 0){
      close(gl_socket[i]);
    }
  }
  exit(0);
  sem_post(&gl_mutex); 

}


